//COPYRIGHT(C) 1998 BY JASON WINNEBECK.  YOU MAY FREELY DISTRIBUTE THIS
//SOURCE AND ACCOMPANYING FILES AS LONG AS YOU STILL KEEP THEM ALL TOGETHER,
//YOU DO NOT MODIFY THEM, AND YOU DON'T CLAIM THEM AS YOUR OWN WORK.

#include "jetfight.h"
#include <jrnd.h>

//                       **************************
//                       ***ENEMY SYSTEM CLASSES***
//                       **************************

int EnemySystem::Init() {
  numbair = 0;
  numbground = 0;

  char c;
  for (c=0; c<10; c++) {
    airkill[c] = 0;
    groundkill[c] = 0;
  }

  return 0;
}

int EnemySystem::TurnTo(int enemy, AirEnemy &ae, float x, float y) {
  tempangle = FindAngle(ae.track.x, ae.track.y, x, y);
  tempdir = FindDirection(ae.track.angle, tempangle);
  if (fabs(tempdir) < ae.stats.turnrate) {
    ae.track.angle = tempangle;
    return 1; //if successful
  } else if (tempdir > 0.0)
    TurnRight(enemy);
  else if (tempdir < 0.0)
    TurnLeft(enemy);

  return 0;
}

int EnemySystem::MoveTo(int enemy, AirEnemy &ae, float x, float y) {
  tempdist = FindDistance(ae.track.x, ae.track.y, x, y);
  TurnTo(enemy, ae, x, y);

  if (tempdist > 192.0) { //if outside 6 squares
    Accelerate(enemy);
  } else {
    Deccelerate(enemy);
    return 1;
  }
  return 0;
}

int EnemySystem::TurnAway(int enemy, AirEnemy &ae, float x, float y) {
  tempangle = FindAngle(x, y, ae.track.x, ae.track.y);
  tempdir = FindDirection(ae.track.angle, tempangle);
  if (fabs(tempdir) < ae.stats.turnrate) {
    ae.track.angle = tempangle;
    return 1; //if already there
  } else if (tempdir > 0.0)
    TurnRight(enemy);
  else if (tempdir < 0.0)
    TurnLeft(enemy);

  return 0;
}

int EnemySystem::MoveAway(int enemy, AirEnemy &ae, float x, float y) {
  tempdist = FindDistance(ae.track.x, ae.track.y, x, y);
  TurnAway(enemy, ae, x, y);

  if (tempdist < 320.0) //if outside 10 squares
    Accelerate(enemy);
  else
    return 1;

  return 0;
}

inline int EnemySystem::MoveToPlayer(int enemy, AirEnemy &ae) {
  return MoveTo(enemy, ae, tempplayer.x, tempplayer.y);
}

inline int EnemySystem::MoveAwayPlayer(int enemy, AirEnemy &ae) {
  return MoveAway(enemy, ae, tempplayer.x, tempplayer.y);
}

inline void EnemySystem::AttackPlayer(int enemy, AirEnemy &ae) {
  if (!ae.temp) {
    if (MoveToPlayer(enemy, ae)) {
      AirShoot(enemy);
      ae.temp = 1;
    }
  } else {
    ae.temp = !MoveAwayPlayer(enemy, ae);
  }
}

inline int EnemySystem::Patrol(int enemy, AirEnemy &ae) {
  return MoveTo(enemy, ae, ae.AI[ae.onpoint].x, ae.AI[ae.onpoint].y);
}

void EnemySystem::RunAI() {
  for (c=0; c < numbair; c++) {
    switch (ae[c].AI[ae[c].onpoint].action) {
      case 1: //move to loc
        if (MoveTo(c, ae[c], ae[c].AI[ae[c].onpoint].x, ae[c].AI[ae[c].onpoint].y))
          ae[c].onpoint++;
        break;
      case 2: //move from loc
        if (MoveAway(c, ae[c], ae[c].AI[ae[c].onpoint].x, ae[c].AI[ae[c].onpoint].y))
          ae[c].onpoint++;
        break;
      case 3: //move to player
        if (MoveToPlayer(c, ae[c]))
          ae[c].onpoint++;
        break;
      case 4: //move away from player
        if (MoveAwayPlayer(c, ae[c]))
          ae[c].onpoint++;
        break;
      case 5: //"attack" player
        AttackPlayer(c, ae[c]);
        break;
      case 6: //"Patrol"
        if (Patrol(c, ae[c]))
          ae[c].onpoint = ae[c].AI[ae[c].onpoint].target; //goto waypoint target
        break;
      case 7: //"Defend area"
        //DefendSpot();
        break;
      case 8: //"Defend enemy"
        //DefendAirEnemy();
        break;
    }
  }
}

void EnemySystem::CalcAll() {
  for (c=0; c < numbair; c++) {
    ae[c].track.x += sin(a2rad*(ae[c].track.angle)) * ae[c].track.speed;
    ae[c].track.y += -cos(a2rad*(ae[c].track.angle)) * ae[c].track.speed;

    if (ae[c].track.x < 0)
      ae[c].track.x = 0;
    else if (ae[c].track.x > unitx - 32)
      ae[c].track.x = unitx - 32;
    if (ae[c].track.y < 0)
      ae[c].track.y = 0;
    else if (ae[c].track.y > unity - 32)
      ae[c].track.y = unity - 32;

    if (ae[c].track.angle < 0.0)
      ae[c].track.angle += 360.0;
    else if (ae[c].track.angle > 360.0)
      ae[c].track.angle -= 360.0;

//Make a Particle
//Oranges from from 160 to 175
  float tempangle = (ae[c].track.angle - 180.0);
  double tempangle2 = tempangle * a2rad;
  float temptrackx = ae[c].track.x + 16.0;
  float temptracky = ae[c].track.y + 16.0;
  for (char c = 0; c < 2; c++)
    Parts.MakeNew(sin(tempangle2) * 16.0 + temptrackx,    //x
                  -cos(tempangle2) * 16.0 + temptracky,   //y
                  tempangle + getrnd(10, -10),            //angle
                  -ae[c].track.speed * 0.3 + getrndf(1.0, -1.0),//speed
                  getrnd(121, 114), 15);                  //color, life
  }
}

void EnemySystem::DisplayAll() {
  for (c=0; c < numbair; c++) {
    Video.DrawObject(ae[c].image, ae[c].track.x, ae[c].track.y, ae[c].track.angle);
    char buff[5];
    itoa(int(ae[c].stats.life), buff, 10);
    Video.ShowText(ae[c].track.x, ae[c].track.y, 15, buff);
  }
  for (c=0; c < numbground; c++)
    Video.DrawObject(ge[c].image, ge[c].track.x, ge[c].track.y, ge[c].track.angle);
}

void EnemySystem::TurnLeft(int enemy) {
  ae[enemy].track.angle -= ae[enemy].stats.turnrate;
}

void EnemySystem::TurnRight(int enemy) {
  ae[enemy].track.angle += ae[enemy].stats.turnrate;
}

void EnemySystem::Accelerate(int enemy) {
  ae[enemy].track.speed += ae[enemy].stats.accel;
  if (ae[enemy].track.speed > ae[enemy].stats.maxspeed)
    ae[enemy].track.speed = ae[enemy].stats.maxspeed;
}

void EnemySystem::Deccelerate(int enemy) {
  ae[enemy].track.speed -= ae[enemy].stats.accel;
  if (ae[enemy].track.speed < ae[enemy].stats.minspeed)
    ae[enemy].track.speed = ae[enemy].stats.minspeed;
}

void EnemySystem::AirShoot(int enemy) {
  if (ae[enemy].ammo.bullets > 0) {
    Sound.MachineGun(128, 2);
    ae[enemy].ammo.bullets -= 8;
    if (ae[enemy].ammo.bullets < 0)
      ae[enemy].ammo.bullets = 0;
    //Make a Bullet
    //Greys from 114 to 121
    Weapons.MakeNewEnemy(sin(ae[enemy].track.angle * a2rad) * 16.0 + ae[enemy].track.x + 16.0, //x
                         -cos(ae[enemy].track.angle * a2rad) * 16.0 + ae[enemy].track.y + 16.0,//y
                         ae[enemy].track.angle, ae[enemy].track.speed + 20,    //angle, speed
                         115, 15, ae[enemy].ammo.bulletdamage);//color, life, damage
  }
}

void EnemySystem::GroundShoot(int enemy) {
}

void EnemySystem::Update(PlayerTrack temp) {
  tempplayer = temp;
  RunAI();
  CalcAll();
  DisplayAll();
}

// ***Weapons Procs***
int EnemySystem::DamageAirEnemies(float x, float y, float damage) { //return 1 if a hit
  for (unsigned char c=0; c < numbair; c++)
    if (FindDistance(x, y, ae[c].track.x, ae[c].track.y) < 16) { //if hit
      //Oranges from from 160 to 175
      for (char d=0; d <= 5; d++) //Make "sparks" from bullet ricochet
        Parts.MakeNew(ae[c].track.x, ae[c].track.y, getrnd(360), getrnd(6, 3), getrnd(175, 160), 15);

      ae[c].stats.life -= damage; //Damage enemy
      if (ae[c].stats.life <= 0) { //If Dead do an explosion
        Sprites.MakeNew(1, ae[c].track.x - (EXPLODE_SIZE - 32) / 2, ae[c].track.y - (EXPLODE_SIZE - 32) / 2);
        Sound.Explosion(128);
        DeleteAirEnemy(c);      //delete this enemy
        Player.EnemyKilled(c);  //Notify Player that this enemy died
      }
      return 1;
    }
  return 0;
}

void EnemySystem::DamageGroundEnemies(float x, float y, float damage) {
}

//         ***EDITOR PROCS***

void EnemySystem::Load(PACKFILE *mapdata) {
  pack_fread(airkill, sizeof(unsigned char) * 10, mapdata);
  pack_fread(groundkill, sizeof(unsigned char) * 10, mapdata);
  pack_fread(&numbair, sizeof(unsigned char), mapdata);
  pack_fread(&numbground, sizeof(unsigned char), mapdata);
  pack_fread(ae, sizeof(AirEnemy) * 50, mapdata);
  pack_fread(ge, sizeof(GroundEnemy) * 50, mapdata);
  for (short c = 0; c < numbair; c++)
    switch (ae[c].type) {
      case 0:
        ae[c].image = a10;
        break;
      case 1:
        ae[c].image = f15;
        break;
      default:
        ae[c].image = a10;
    }
  for (short c = 0; c < numbground; c++)
    switch (ge[c].type) {
      default:
        ae[c].image = turret1;
    }
}

void EnemySystem::Save(PACKFILE *mapdata) {
  pack_fwrite(airkill, sizeof(unsigned char) * 10, mapdata);
  pack_fwrite(groundkill, sizeof(unsigned char) * 10, mapdata);
  pack_fwrite(&numbair, sizeof(unsigned char), mapdata);
  pack_fwrite(&numbground, sizeof(unsigned char), mapdata);
  pack_fwrite(ae, sizeof(AirEnemy) * 50, mapdata);
  pack_fwrite(ge, sizeof(GroundEnemy) * 50, mapdata);
}

void EnemySystem::AddAirEnemy(int type, AirEnemyTrack track, EnemyAmmo ammo) {
  //make a new AI script
  AIScript temp[10];
  temp[0].action = 5;
  temp[0].target = 0;
  temp[0].x = 0.0;
  temp[0].y = 0.0;

  for (char c=1; c < 10; c++) {
    temp[c].action = 0;
    temp[c].target = 0;
    temp[c].x = 0.0;
    temp[c].y = 0.0;
  }

  AddAirEnemy(type, track, ammo, temp);
}

void EnemySystem::AddAirEnemy(int type, AirEnemyTrack track, EnemyAmmo ammo, AIScript AI[10]) {
  int c;
  ae[numbair].type = type;
  ae[numbair].track = track;
  ae[numbair].ammo = ammo;
  for (c = 0; c < 10; c++)
    ae[numbair].AI[c] = AI[c];
  ae[numbair].onpoint = 0;
  ae[numbair].temp = 0;

  switch (type) {
    case 0: //a10
      ae[numbair].image = a10;
      ae[numbair].stats.turnrate = 6.0;
      ae[numbair].stats.life = 100.0;
      ae[numbair].stats.accel = 0.5;
      ae[numbair].stats.minspeed = 4.0;
      ae[numbair].stats.maxspeed = 16.0;
      ae[numbair].ammo.bulletdamage = 40.0;
      ae[numbair].ammo.bombdamage = 50.0;
      ae[numbair].ammo.missiledamage = 100.0;
      break;
    case 1: //f15
      ae[numbair].image = f15;
      ae[numbair].stats.turnrate = 12.0;
      ae[numbair].stats.life = 40.0;
      ae[numbair].stats.accel = 1.0;
      ae[numbair].stats.minspeed = 6.0;
      ae[numbair].stats.maxspeed = 32.0;
      ae[numbair].ammo.bulletdamage = 20.0;
      ae[numbair].ammo.bombdamage = 10.0;
      ae[numbair].ammo.missiledamage = 120.0;
      break;
    default: //a10 if nothing else
      ae[numbair].image = a10;
      ae[numbair].stats.turnrate = 6.0;
      ae[numbair].stats.life = 100.0;
      ae[numbair].stats.accel = 0.5;
      ae[numbair].stats.minspeed = 4.0;
      ae[numbair].stats.maxspeed = 16.0;
      ae[numbair].ammo.bulletdamage = 40.0;
      ae[numbair].ammo.bombdamage = 50.0;
      ae[numbair].ammo.missiledamage = 100.0;
      break;
  }

  ae[numbair].track.speed = ae[numbair].stats.minspeed;

  numbair++;
}

void EnemySystem::AddGroundEnemy(int type, GroundEnemyTrack track, EnemyAmmo ammo) {
  switch (type) {
    case 1:
      ge[numbground].type = type;
      ge[numbground].image = NULL;
      ge[numbground].track = track;
      ge[numbground].ammo = ammo;
      break;
  }

  numbground++;
}

void EnemySystem::DeleteAirEnemy(char t) {
  char c;
  for (c=t; c < numbair - 1; c++)
    ae[c] = ae[c+1];

  numbair--;
}

void EnemySystem::DeleteGroundEnemy(char t) {
  char c;
  for (c=t; c < numbground - 1; c++)
    ge[c] = ge[c+1];

  numbground--;
}

void EnemySystem::SetKills(unsigned char ak[10], unsigned char gk[10]) {
  char c;
  for (c = 0; c < 10; c++) {
    airkill[c] = ak[c];
    groundkill[c] = gk[c];
  }
}

void EnemySystem::GetKills(unsigned char ak[10], unsigned char gk[10]) {
  char c;
  for (c = 0; c < 10; c++) {
    ak[c] = airkill[c];
    gk[c] = groundkill[c];
  }
}

void EnemySystem::Draw(BITMAP *BUFFER, short int viewx, short int viewy) {
  int x, y;
  for (c=0; c < numbair; c++) {
    x = int(ae[c].track.x);
    y = int(ae[c].track.y);
    if (x > viewx - 32 && x < viewx + mscrx && y > viewy - 32 && y < viewy + mscry) {
      tempangle = ae[c].track.angle * 256.0 / 360.0;
      rotate_sprite(BUFFER, (BITMAP *)data[ae[c].image].dat, x - viewx, y - viewy, ftofix(tempangle));
    }
  }
  for (c=0; c < numbground; c++) {
    x = int(ge[c].track.x);
    y = int(ge[c].track.y);
    if (x > viewx - 32 && x < viewx + mscrx && y > viewy - 32 && y < viewy + mscry) {
      tempangle = ge[c].track.angle * 256.0 / 360.0;
      rotate_sprite(BUFFER, (BITMAP *)data[ge[c].image].dat, x - viewx, y - viewy, ftofix(tempangle));
    }
  }
  //ANGLE IN 360 DEGREE FORMAT!!!
}

// ****GET DATAS****

char EnemySystem::GetAirEnemy(short x, short y, AirEnemy &ret) { //returns air enemy at x,y
  short int x2, y2;
  x2 = short(x) * 32;
  y2 = short(y) * 32;
  for (char c=0; c < numbair; c++)
    if ((ae[c].track.x == x2) && (ae[c].track.y == y2)) {
      ret = ae[c];
      return c;
    }

  return -1;
}

char EnemySystem::GetGroundEnemy(short x, short y, GroundEnemy &ret) {
  short int x2, y2;
  x2 = short(x) * 32;
  y2 = short(y) * 32;
  for (char c=0; c < numbground; c++)
    if ((ge[c].track.x == x2) && (ge[c].track.y == y2)) {
      ret = ge[c];
      return c;
    }

  return -1;
}
