//COPYRIGHT(C) 1998 BY JASON WINNEBECK.  YOU MAY FREELY DISTRIBUTE THIS
//SOURCE AND ACCOMPANYING FILES AS LONG AS YOU STILL KEEP THEM ALL TOGETHER,
//YOU DO NOT MODIFY THEM, AND YOU DON'T CLAIM THEM AS YOUR OWN WORK.

#include "jetfight.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "tdgui.h"
#include <jrnd.h>

//Function Declarations
int MapEnter();         //Enters 1024x768 grfx mode & loads editor
void MapExit();         //Destructs and clean exits
int Quit();             //asks user if he/she wants to quit
void UpdateScreen();    //Sends BUFFER to screen
void UpdateControls();  //Checks keypresses
void DrawGrid();        //Draws 32x32 grids to BUFFER
void MoveView(short int x, short int y);//Moves view coords
void SetView(short int x, short int y); //Sets view coords

void AskSave();         //asks if user wants to save or not

void AddTile(short int x, short int y, unsigned char image);
                        //Adds a tile to the WorldSystem
int PickTile();         //Pick a tile to lay down

volatile char upd = 0;       //UPDPROC
void updproc(...) {upd = 1;} //cycles palette(s)
END_OF_FUNCTION(updproc);

//Variable Declarations
#define NUM_BUFFS 5       //number of BUFFS
#define version   ".04a"  //Map Editor Version
#define maximage  130
BITMAP *BUFFER;           //BUFFER
BITMAP *temp;             //temp bitmap for mouse placement
short int viewx, viewy;   //Where is the viewport located on the map?
char cont = 1;            //Continue loop?
char crand;               //random to add for tile laying

//Temp Vars
unsigned char update = 1; //do we need to update?
char changed = 0;         //Has the map been altered?
RGB waterpal;             //Water cycling palletes
char wateradd;
unsigned char currimage;  //current image the mouse lays w/LMB
char title[40];           //for the title of a window
char buffs[NUM_BUFFS][40] = {//For text inputs
  "", "", "", "", ""};
AIScript aitemp[10];      //temporary AIScript
short int whereclickx;    //where did our mouse click (in map coords)
short int whereclicky;
short int prevx, prevy;   //Where was the last spot?
                          // (used in LayTile)

//Map Vars
unsigned char airkill[10];
unsigned char groundkill[10];
NavSystemData navdata[10];//Player's navdata
char numpoints;           //Player's number of waypoints
unsigned char midinum;

//Menuing Procs
void ClearBuffs();        //clears the buffs array
void DoMainMenu();        //Does the mainmenu

int GetNum(char *prompt, int old, int low, int high);
                          //gets a number between low and high

int AddAirEnemy();        //adds enemy to the map
int AddGroundEnemy();
int AddWaypoint();        //adds a waypoint
int DeleteAirEnemy();     //deletes an enemy from the map
int DeleteGroundEnemy();
int EditAirEnemy();       //Edits an enemy
int EditGroundEnemy();
int EditPlayer();         //Edits the player

int MovePlayer();         //sets player pos to the click spot
int GotoPlayer();         //sets view to the player
int GotoLocation();       //goes to map location x,y

int EditMapProp();        //Edits properties of the entire map
int ShowHelp();           //Shows help file
int NewMap();             //starts a new map
int Load();               //saves and loads maps
int Save();

inline int StartMusic() {World.StartMusic(); return D_CLOSE;} //Plays map music
inline int StopMusic() {play_midi(NULL, 0); return D_CLOSE;}  //Stops map music
inline int ExitMenu() {return D_CLOSE;} //Exits the menu

//Dialog Procs
int d_check_ai(int msg, DIALOG *d, int c); //checks the AI waypoint list and
                                           //changes the box data when the
                                           //waypoint select changes

#include "mapedit.h"  //dialogs and menu declarations

// ******MAIN FUNCTION
int MapEdit() {
  if (MapEnter())
    return 1;

  do {
    UpdateControls();
    UpdateScreen();
  } while (cont);

  AskSave();

  MapExit();

  return 0;
}
// ******END OF MAIN FUNCTION

int MapEnter() {
  midinum = 1;
  cont = 1;
  update = 1;
  changed = 0;
  upd = 1;

  waterpal.r = 0;
  waterpal.g = 0;
  waterpal.b = 32;
  wateradd = 1;

  currimage = water1;
  crand = 6;

  for (char c=0; c < 10; c++) {
    navdata[c].type = 0;
    navdata[c].targ = 0;
    navdata[c].x = 0;
    navdata[c].y = 0;
  }
  numpoints = 0;

  set_color_depth(8);
  if (set_gfx_mode(GFX_AUTODETECT, mscrx, mscry, 0, 0) == -1)
    return 1;

  World.Init();

  numpoints = Player.GetNumbPoints();

  set_palette((RGB *)data[jetfight].dat);
  clear(screen);

  BUFFER = create_bitmap(mscrx, mscry);
  clear(BUFFER);
  temp = create_bitmap(200, 12);
  clear(temp);

  set_mouse_sprite(NULL);
  show_mouse(screen);

  SetView(0, 0);
  UpdateScreen();

  billalert("Welcome to:", "JetFight map editor version:", version, "&OK", NULL, 'o', NULL);
  Sound.MouseClick(128);

  LOCK_FUNCTION(updproc);
  LOCK_VARIABLE(upd);

  install_int(updproc, 35);

  return 0;
}

void MapExit() {
  play_midi(NULL, 0);
  remove_int(updproc);

  destroy_bitmap(BUFFER);
  destroy_bitmap(temp);

  if (Video.EnterGraphics(32))
    Video.EnterGraphics(24);
}

int Quit() {
  Sound.Neenee(128);
  if (billalert("Quit", "Are you sure you wish to", "quit JetFight map editor?", "&OK", "Cancel", 'o', 27) == 1) {
    Sound.MouseClick(128);
    cont = 0;
  } else {
    Sound.MouseClick(128);
    cont = 1;
  }

  return D_CLOSE;
}

void UpdateScreen() {
  if (update) {
    text_mode(-1);
    World.Draw(BUFFER, viewx, viewy);
    DrawGrid();
    textprintf(BUFFER, font, 0, 0, 15, "Viewx: %i-%i, Viewy: %i-%i", viewx, viewx + mscrx, viewy, viewy + mscry);
    textprintf(BUFFER, font, 0, 8, 15, "Mapx: %i-%i, Mapy: %i-%i", viewx / 32, (viewx + mscrx) / 32 - 1, viewy / 32, (viewy + mscry) / 32 - 1);
    textprintf_centre(BUFFER, font, mscrx / 2, mscry - 8, 15, "ALT for menu, P to pick a tile");
    scare_mouse();
    blit(BUFFER, screen, 0, 0, 0, 0, mscrx, mscry);
    unscare_mouse();
    blit(BUFFER, temp, 0, 16, 0, 0, 200, 12);
    textprintf(screen, font, 0, 16, 15, "Mousex: %i, Mousey: %i", (mouse_x + viewx) / 32, (mouse_y + viewy) / 32);

    update = 0;
  }

  if (upd) {
    blit(temp, screen, 0, 0, 0, 16, 200, 12);
    textprintf(screen, font, 0, 16, 15, "Mousex: %i, Mousey: %i", (mouse_x + viewx) / 32, (mouse_y + viewy) / 32);

    waterpal.b += wateradd;
    waterpal.g += wateradd;
    waterpal.r += wateradd;
    if (waterpal.b > 62)
      wateradd = -1;
    else if (waterpal.b < 33)
      wateradd = 1;
    set_color(111, &waterpal);

    upd = 0;
  }
}

void UpdateControls() {
  // ***CHECK KEYBOARD FOR MOVEMENT
  if      (key[KEY_DOWN])
    MoveView(0, 8);
  else if (key[KEY_UP])
    MoveView(0, -8);
  if      (key[KEY_LEFT])
    MoveView(-8, 0);
  else if (key[KEY_RIGHT])
    MoveView(8, 0);

  //Check for fast movement
  if      (key[KEY_DOWN] && (key[KEY_RSHIFT] || key[KEY_LSHIFT]))
    MoveView(0, 40);
  else if (key[KEY_UP] && (key[KEY_RSHIFT] || key[KEY_LSHIFT]))
    MoveView(0, -40);
  if      (key[KEY_LEFT] && (key[KEY_RSHIFT] || key[KEY_LSHIFT]))
    MoveView(-40, 0);
  else if (key[KEY_RIGHT] && (key[KEY_RSHIFT] || key[KEY_LSHIFT]))
    MoveView(40, 0);

  // ***CHECK KEYBOARD FOR INTERFACE COMMANDS
  if (key[KEY_S])         //Save
    Save();
  else if (key[KEY_L])    //Load
    Load();
  else if (key[KEY_N])    //New Map
    NewMap();
  else if (key[KEY_P])    //Pick Tile
    PickTile();
  else if (key[KEY_M])    //Map Properties
    EditMapProp();
  else if (key[KEY_F1])   //Help
    ShowHelp();
  else if (key[KEY_Q])    //Quit
    Quit();
  else if (key[KEY_ALT] || (mouse_y == 0)) //Main Menu
    DoMainMenu();

  // ***CHECK MOUSE FOR EDITING COMMANDS
  if (mouse_b & 1) {      //Lay Tile
    LayTiles((mouse_x + viewx) / 32, (mouse_y + viewy) / 32, currimage);
    update = 1;
    changed = 1;
/*    if (crand)
      AddTile((mouse_x + viewx) / 32, (mouse_y + viewy) / 32, currimage + getrnd(crand));
    else
      AddTile((mouse_x + viewx) / 32, (mouse_y + viewy) / 32, currimage);*/
  } else if (mouse_b & 2) { //Edit Properties
    update = 1;
    whereclickx = (mouse_x + viewx) / 32;
    whereclicky = (mouse_y + viewy) / 32;
    do_billmenu(RightClickMenu, mouse_x, mouse_y);
  }
}

void DrawGrid() {
  int c;
  for (c = -(viewx % 32); c < mscrx; c += 32)
    vline(BUFFER, c, 0, mscry, 8);
  for (c = -(viewy % 32); c < mscry; c += 32)
    hline(BUFFER, 0, c, mscrx, 8);        
}

void MoveView(short int x, short int y) {
  SetView(viewx + x, viewy + y);
}

void SetView(short int x, short int y) {
  update = 1;

  viewx = x;
  viewy = y;

  if (viewx < 0)
    viewx = 0;
  if (viewy < 0)
    viewy = 0;
  if (viewx > unitx - mscrx)
    viewx = unitx - mscrx;
  if (viewy > unity - mscry)
    viewy = unity - mscry;
}

void AskSave() {
  if (changed)
    if (billalert("Exit", "You have not saved this", "map.  Save Now?", "&Yes", "&No", 'y', 'n') == 1) {
      Sound.MouseClick(128);
      Save();
    } else
      Sound.MouseClick(128);

  changed = 0;
}

void AddTile(short int x, short int y, unsigned char image) {
  update = 1;
  changed = 1;
  World.AddTile(x, y, image);
}

int Load() {
  AskSave();

  char filename[80];
  strcpy(filename, "");
  if (billfile_select("Load Map", filename, "SAV")) {
    strlwr(filename);
    Sound.MouseClick(128);
    PACKFILE *mapdata = pack_fopen(filename, F_READ_PACKED);
    if (World.Load(mapdata)) {
      billalert("File Error!", "Current version of", "file not compatable!", "&OK", NULL, 'o', NULL);
      World.Init();
    }
    pack_fclose(mapdata);

    update = 1;

    Enemies.GetKills(airkill, groundkill);
    numpoints = Player.GetNumbPoints();
  } else {
    Sound.MouseClick(128);
  }

  return D_CLOSE;
}

int Save() {
  char filename[80];
  strcpy(filename, "");
  if (billfile_select("Save Map", filename, "SAV")) {
    strlwr(filename);
    Sound.MouseClick(128);
    PACKFILE *mapdata = pack_fopen(filename, F_WRITE_PACKED);
    World.Save(mapdata);
    pack_fclose(mapdata);

    changed = 0;
  } else {
    Sound.MouseClick(128);
  }

  return D_CLOSE;
}

int NewMap() {
  Sound.Neenee(128);
  if (billalert("New Map", "Are you sure you want", "to start a new map?", "&Yes", "&No", 'y', 'n') == 1) {
    Sound.MouseClick(128);

    AskSave();

    World.Init();
    update = 1;
  } else {
    Sound.MouseClick(128);
  }

  return D_CLOSE;
}

int PickTile() {
  update = 1;
  if (Video.EnterGraphics(8)) {
    currimage = water1;
    return D_O_K;
  }

  show_mouse(screen);
  scare_mouse();

  int x, y;
  for (y = 0; y < scry / 32; y ++) //Show the images
    for (x = 0; x < scrx / 32; x ++)
      if (y * scrx / 32 + x + 1 <= maximage)
        blit((BITMAP *)data[AtoD((y*scrx/32)+x+1)].dat, screen, 0, 0, x*32, y*32, 32, 32);

  int c;
  for (c = 0; c < scrx; c += 32) //Draw the grid
    vline(screen, c, 0, scry, 8);
  for (c = 0; c < scry; c += 32)
    hline(screen, 0, c, scrx, 8);

  unscare_mouse();

  do {
  } while(!(mouse_b & 1) && !key[KEY_ESC]);

  if (!key[KEY_ESC]) {
    x = mouse_x / 32;
    y = mouse_y / 32;

    currimage = AtoD((y * scrx / 32) + x + 1);

    switch (currimage) {
      case water1: //for fill tiles
      case beach1:
      case grass1:
//      case jungle1:
        crand = 6;
        break;
      case workfast: //for NULL tile
        crand = 0;
        break;
      default: //for border tiles
        crand = 3;
        break;
    }
  }

  set_color_depth(8);
  set_gfx_mode(GFX_AUTODETECT, mscrx, mscry, 0, 0);
  set_pallete((RGB *)data[jetfight].dat);

  show_mouse(screen);

  return D_O_K;
}

// ***************************
// ****** MENUING PROCS ******
// ***************************

void ClearBuffs() {
  for (int c = 0; c < NUM_BUFFS; c++)
    strcpy(buffs[c], "");
}

void DoMainMenu() {
  ret = -1;
  clear_keybuf();
  do_dialog(main_menu_dialog, 0);
  update = 1;
}

int GetNumb(char *prompt, int old, int low, int high) {
  ClearBuffs();
  ret = -1;
  //set up prompt buffers
  strcpy(buffs[0], prompt);      //passed prompt

  sprintf(buffs[1], "Between %i and %i", low, high);

  itoa(old, buffs[2], 10);       //old value in box

  //do dialog
  centre_dialog(getnumb);
  init_moveable(getnumb);
  do_dialog(getnumb, 0);
  shutdown_moveable(getnumb);

  if (ret == 1) {                //if ok'd
    int temp = atoi(buffs[2]);
    if (temp > high)
      temp = high;
    else if (temp < low)
      temp = low;

    return temp;
  } else
    return old;
}

int AddAirEnemy() {
  ret = -1;
  ClearBuffs();
  sprintf(title, "Adding air enemy %i of 50", Enemies.GetNextAirEnemyNumb());

  centre_dialog(addairenemy);
  init_moveable(addairenemy);
  do_dialog (addairenemy, 0);
  shutdown_moveable(addairenemy);

  AirEnemy tempair;

  if (ret) {
    //save the data before being erased by the AI scripter
    tempair.ammo.bullets = atoi(buffs[0]);
    tempair.ammo.bombs = atoi(buffs[1]);
    tempair.ammo.missiles = atoi(buffs[2]);
    tempair.type = addairenemy[airlist].d1;
    tempair.track.x = whereclickx * 32;
    tempair.track.y = whereclicky * 32;
    tempair.track.angle = atoi(buffs[3]);
  }

  switch (ret) {
    case 1: //Make default AI script
      Enemies.AddAirEnemy(tempair.type, tempair.track, tempair.ammo);
      changed = 1;
      break;
    case 2: //Go to script editor
      ClearBuffs();
      ret = -1;
      strcpy(title, "Editing AI script");
      //make a new AI script
      aitemp[0].action = 5;
      aitemp[0].target = 0;
      aitemp[0].x = 0.0;
      aitemp[0].y = 0.0;

      for (char c=1; c < 10; c++) {
        aitemp[c].action = 0;
        aitemp[c].target = 0;
        aitemp[c].x = 0.0;
        aitemp[c].y = 0.0;
      }
      centre_dialog(editaiscript);
      init_moveable(editaiscript);
      do_dialog(editaiscript, 0);
      shutdown_moveable(editaiscript);

      if (ret == 1) {
        //add the enemy with AI
        Enemies.AddAirEnemy(tempair.type, tempair.track, tempair.ammo, aitemp);
        changed = 1;
      } else {
        //never mind AI -- default AI
        Enemies.AddAirEnemy(tempair.type, tempair.track, tempair.ammo);
        changed = 1;
      }
      break;
  }

  return D_CLOSE;
}

int AddGroundEnemy() {
  ret = -1;
  return D_CLOSE;
}

int AddWaypoint() {
  if (numpoints >= 10) {
    billalert("Error", "You already have", "10 waypoints!", "&OK", NULL, 'o', NULL);
  } else {
    AirEnemy dummy;
    char temp = Enemies.GetAirEnemy(whereclickx, whereclicky, dummy);
    if (temp != -1) {
      navdata[numpoints].type = 2;
      navdata[numpoints].targ = temp;
    } else {
      navdata[numpoints].type = 0;
      navdata[numpoints].x = whereclickx * 32;
      navdata[numpoints].y = whereclicky * 32;
    }
    numpoints++;
    billalert("Done!", "Waypoint", "Added", "&OK", NULL, 'o', NULL);

    Player.SetNavData(navdata, numpoints);
  }

  return D_CLOSE;
}

int DeleteAirEnemy() {
  AirEnemy dummy;
  char temp = Enemies.GetAirEnemy(whereclickx, whereclicky, dummy);
  if (temp != -1)
    Enemies.DeleteAirEnemy(temp);
  else
    billalert("Fatal bug detected in user!", "Does that look like an AIR enemy to", "you?  I didn't think so!", "&Sorry!", NULL, 's', NULL);
  return D_CLOSE;
}

int DeleteGroundEnemy() {
  GroundEnemy dummy;
  char temp = Enemies.GetGroundEnemy(whereclickx, whereclicky, dummy);
  if (temp != -1)
    Enemies.DeleteGroundEnemy(temp);
  else
    billalert("Fatal bug detected in user!", "Does that look like a GROUND enemy", "to you?  I didn't think so!", "&Sorry!", NULL, 's', NULL);
  return D_CLOSE;
}

int EditAirEnemy() {
  //set up vars
  ret = -1;
  char tempindex;
  AirEnemy tempair;
  tempindex = Enemies.GetAirEnemy(whereclickx, whereclicky, tempair);

  if (tempindex != -1) {
    //make title
    sprintf(title, "Editing Enemy %i", tempindex);

    //Send data to window
    ClearBuffs();
    itoa(tempair.ammo.bullets, buffs[0], 10);
    itoa(tempair.ammo.bombs, buffs[1], 10);
    itoa(tempair.ammo.missiles, buffs[2], 10);
    itoa(int(tempair.track.angle), buffs[3], 10);
    addairenemy[airlist].d1 = tempair.type;

    //do window
    centre_dialog(addairenemy);
    init_moveable(addairenemy);
    do_dialog (addairenemy, 0);
    shutdown_moveable(addairenemy);

    if (ret) {
      //save the data before being erased by the AI scripter
      tempair.ammo.bullets = atoi(buffs[0]);
      tempair.ammo.bombs = atoi(buffs[1]);
      tempair.ammo.missiles = atoi(buffs[2]);
      tempair.type = addairenemy[airlist].d1;
      tempair.track.x = whereclickx * 32;
      tempair.track.y = whereclicky * 32;
      tempair.track.angle = atoi(buffs[3]);
    }

    switch (ret) {
      case 1: //Make default AI script
        Enemies.ChangeAirEnemy(tempindex, tempair);
        changed = 1;
        break;
      case 2: //Go to script editor
        Enemies.ChangeAirEnemy(tempindex, tempair); //save changes data so far

        //Load dialog
        ClearBuffs();
        ret = -1;
        aitemp = tempair.AI;
        strcpy(title, "Editing AI script");

        //do dialog
        centre_dialog(editaiscript);
        init_moveable(editaiscript);
        do_dialog(editaiscript, 0);
        shutdown_moveable(editaiscript);

        if (ret == 1) {
          //if OK'd then update
          tempair.AI = aitemp;
          Enemies.ChangeAirEnemy(tempindex, tempair);
          changed = 1;
        }
        break;
    }
  } else //if it wasn't found
    billalert("Error!", "Enemy not found!", NULL, "&OK", NULL, 'o', NULL);

  return D_CLOSE;
}

int EditGroundEnemy() {
  ret = -1;
  return D_CLOSE;
}

int EditPlayer() {
  ret = -1;

  //Get Player Data
  PlayerTrack track;
  PlayerAmmo ammo;
  NavSystemData navdata[10];
  unsigned char waypoints;
  unsigned char type;

  Player.GetData(track, ammo, type, navdata, waypoints);

  //Send data to window
  ClearBuffs();
  itoa(ammo.bullets, buffs[0], 10);
  itoa(ammo.bombs, buffs[1], 10);
  itoa(ammo.missiles, buffs[2], 10);
  itoa(int(track.angle), buffs[3], 10);
  editplayer[playerlist].d1 = type;

  //Do Dialog
  centre_dialog(editplayer);
  init_moveable(editplayer);
  do_dialog(editplayer, 0);
  shutdown_moveable(editplayer);

  if (ret == 1) { //'ok'
    //Save Player Data and exit
    ammo.bullets = atoi(buffs[0]);
    ammo.bombs = atoi(buffs[1]);
    ammo.missiles = atoi(buffs[2]);
    type = editplayer[airlist].d1;
    track.angle = atoi(buffs[3]);
    Player.SetData(track, ammo, type, navdata, waypoints);
  } else if (ret == 2) { //'Edit Nav points'
    //Waypoint editor dialog

    //Save Player Data
    ammo.bullets = atoi(buffs[0]);
    ammo.bombs = atoi(buffs[1]);
    ammo.missiles = atoi(buffs[2]);
    type = editplayer[playerlist].d1;
    track.angle = atoi(buffs[3]);
    Player.SetData(track, ammo, type, navdata, waypoints);
    changed = 1;
  }

  return D_CLOSE;
}

int MovePlayer() {
  Player.SetPos(whereclickx, whereclicky);
  changed = 1;
  return D_CLOSE;
}

int GotoPlayer() {
  PlayerTrack temp = Player.GetTrack();
  SetView(int(temp.x), int(temp.y));
  return D_CLOSE;
}

int GotoLocation() {
  short x = GetNumb("Enter MapX:", 0, 0, 255) * 32;
  short y = GetNumb("Enter MapY:", 0, 0, 255) * 32;
  SetView(x, y);
  return D_CLOSE;
}

int EditMapProp() {
  ret = -1;
  mapprop[midilist].d1 = World.GetMusic();

  centre_dialog(mapprop);
  init_moveable(mapprop);
  do_dialog (mapprop, 0);
  shutdown_moveable(mapprop);

  if (ret > 0) {
    World.ChangeMusic(mapprop[midilist].d1);
    changed = 1;
  }

  return D_CLOSE;
}

int ShowHelp() {
  Sound.Neenee(128);
  billalert("Help", "Sorry, help file is", "not yet available.", "&OK", NULL, 'o', NULL);
  Sound.MouseClick(128);

  return D_CLOSE;
}

// ***DIALOG PROCEDURES

int d_check_ai(int msg, DIALOG *d, int c) {
  static char onpoint; //what ai point are we on?

  switch (msg) {
    case MSG_START: //initialize
      onpoint = 0;
      editaiscript[ailist].d1 = 0;
      editaiscript[actionlist].d1 = aitemp[0].action;
      itoa(aitemp[0].target, buffs[0], 10);
      itoa(int(aitemp[0].x) / 32, buffs[1], 10);
      itoa(int(aitemp[0].y) / 32, buffs[2], 10);
      break;
    case MSG_IDLE: //when we're free, check the list
      if (editaiscript[ailist].d1 != onpoint) { //if the waypoint changed
        //save changed data
        aitemp[onpoint].action = editaiscript[actionlist].d1;
        aitemp[onpoint].target = atoi(buffs[0]);
        aitemp[onpoint].x = float(atoi(buffs[1]) * 32);
        aitemp[onpoint].y = float(atoi(buffs[2]) * 32);

        //change what point we're on
        onpoint = editaiscript[ailist].d1;

        //switch out data
        editaiscript[actionlist].d1 = aitemp[onpoint].action;
        itoa(aitemp[onpoint].target, buffs[0], 10);
        itoa(int(aitemp[onpoint].x) / 32, buffs[1], 10);
        itoa(int(aitemp[onpoint].y) / 32, buffs[2], 10);

        return D_REDRAW; //update the dialog
      }
      break;
    case MSG_END: //Update the latest data!
      aitemp[onpoint].action = editaiscript[actionlist].d1;
      aitemp[onpoint].target = atoi(buffs[0]);
      aitemp[onpoint].x = float(atoi(buffs[1]) * 32);
      aitemp[onpoint].y = float(atoi(buffs[2]) * 32);

      return D_O_K;
  }

  return D_O_K;
}
