//COPYRIGHT(C) 1998 BY JASON WINNEBECK.  YOU MAY FREELY DISTRIBUTE THIS
//SOURCE AND ACCOMPANYING FILES AS LONG AS YOU STILL KEEP THEM ALL TOGETHER,
//YOU DO NOT MODIFY THEM, AND YOU DON'T CLAIM THEM AS YOUR OWN WORK.

#include <stdio.h>
#include "jrnd.h"
#include "jetfight.h"

//                       **************************
//                       ***SOUND SYSTEM CLASSES***
//                       **************************

SoundSystem::SoundSystem() {  //Constructor Loads all sound data
  //sets loop statuses
  jetloop = 0;
  gunloop = 0;
}

int SoundSystem::Init() { //loads sounds and sets up sfx system
  printf("  Installing Soundcard. . .\n");
  if (install_sound(DIGI_AUTODETECT, MIDI_AUTODETECT, "") == -1) {
    printf("**Error installing sound drivers!!!\n%s\n", allegro_error);
    return 1;
  }

  randomize();

  return 0;
}

void SoundSystem::Applause(int pan) { //plays Applause sound
  play_sample((SAMPLE *)data[clap].dat, vol, pan, 1000, 0);
}

void SoundSystem::AutopilotOn(int pan) { //plays Autopilot on
  play_sample((SAMPLE *)data[autoon].dat, vol, pan, 1000, 0);
}

void SoundSystem::Blip(int pan) { //plays blip/clicking sound
  play_sample((SAMPLE *)data[blip].dat, vol, pan, 1000, 0);
}

void SoundSystem::Bomb(int pan) { //plays falling bomb sound
  play_sample((SAMPLE *)data[bomb2].dat, vol, pan, 1000, 0);
}

void SoundSystem::Cannon(int pan) { //plays cannon sound
  play_sample((SAMPLE *)data[multgun].dat, vol, pan, 1000, 0);
}

void SoundSystem::Explosion(int pan) { //plays a random explosion sound
  if (getrnd(1,0) == 1)
    play_sample((SAMPLE *)data[explode].dat, vol, pan, 1000, 0);
  else
    play_sample((SAMPLE *)data[explode2].dat, vol, pan, 1000, 0);
}

void SoundSystem::MultExplosion(int pan, int length) { //plays mult expl snds
  multleft = length - 1;
  multpan = pan;

//  stop_sample((SAMPLE *)data[explode].dat);
//  stop_sample((SAMPLE *)data[explode2].dat);

  Explosion(pan);

  multexp = getrnd(10, 4);
}

void SoundSystem::Jet(int pan, int cmd, int freq) { //plays the engine sounds for the jet
  if (cmd == 0) {
    jetloop = 0;
    stop_sample((SAMPLE *)data[jet].dat);
  }
  else
    if (jetloop == 0) {
      jetloop = 1;
      play_sample((SAMPLE *)data[jet].dat, vol, pan, freq, 1);
    }
    else
      adjust_sample((SAMPLE *)data[jet].dat, vol, pan, freq, 1);
}

void SoundSystem::Laser(int pan) { //plays a laser sound effect
  play_sample((SAMPLE *)data[lazer].dat, vol, pan, 1000, 0);
}

void SoundSystem::MachineGun(int pan, int cmd) { //plays a machine gun sound
  if (cmd == 0) {
    gunloop = 0;
    stop_sample((SAMPLE *)data[machineg].dat);
  } else if (cmd == 1) {
    if (gunloop == 0){
      gunloop = 1;
      play_sample((SAMPLE *)data[machineg].dat, vol, pan, 1000, 1);
    }
    else
      adjust_sample((SAMPLE *)data[machineg].dat, vol, pan, 1000, 1);
  } else
    play_sample((SAMPLE *)data[machineg].dat, vol, pan, 1000, 0);
}

void SoundSystem::Missile(int pan) { //plays missile sound
  play_sample((SAMPLE *)data[missile].dat, vol, pan, 1000, 0);
}

void SoundSystem::MouseClick(int pan) { //plays mouse clicking sound
  play_sample((SAMPLE *)data[mouse].dat, vol, pan, 1000, 0);
}

void SoundSystem::NavSystemOn(int pan) { //plays "Nav System On" sound
  play_sample((SAMPLE *)data[navon].dat, vol, pan, 1000, 0);
}

void SoundSystem::Neenee(int pan) {
  play_sample((SAMPLE *)data[neenee].dat, vol, pan, 1000, 0);
}

void SoundSystem::SmallGunShot(int pan) { //plays handgun sound
  play_sample((SAMPLE *)data[bangbang].dat, vol, pan, 1000, 0);
}

void SoundSystem::Welcome(int pan) { //plays voice, "Welcome."
  play_sample((SAMPLE *)data[welcome].dat, vol, pan, 1000, 0);
}

void SoundSystem::PlayMusic(int music) { //plays MIDI
  play_midi((MIDI *)data[music].dat, 1);
}

void SoundSystem::MusicStop() { //stops any music playing
  play_midi(NULL, 0);
}

void SoundSystem::Update() { //update SoundSystem
  if (multexp)
    multexp--;
  else if (multleft)
    MultExplosion(multpan, multleft);
}
