//COPYRIGHT(C) 1998 BY JASON WINNEBECK.  YOU MAY FREELY DISTRIBUTE THIS
//SOURCE AND ACCOMPANYING FILES AS LONG AS YOU STILL KEEP THEM ALL TOGETHER,
//YOU DO NOT MODIFY THEM, AND YOU DON'T CLAIM THEM AS YOUR OWN WORK.

#include <jrnd.h>
#include "jetfight.h"

char CurrFileVer = '1';

//                       **************************
//                       ***WORLD SYSTEM CLASSES***
//                       **************************

int WorldSystem::Init() { //loads world
  miditoplay = 1;

  NavSystemData navdata[10];
  PlayerTrack track;
  track.x = 320.0;
  track.y = 320.0;
  track.angle = 90.0;
  track.speed = 0.0;

  PlayerAmmo ammo;
  ammo.bullets = 10000;
  ammo.bombs = 5;
  ammo.missiles = 3;

  if (Player.Init(0, track, navdata, ammo, 0))
    return 1;
  if (Enemies.Init())
    return 1;
  int x, y;
  for (x = 0; x < mapx; x++) {
    for (y = 0; y < mapy; y++) {
      switch (getrnd(6) + 1) {
        case 1:
          map[x][y].image = water1;
          break;
        case 2:
          map[x][y].image = water2;
          break;
        case 3:
          map[x][y].image = water3;
          break;
        case 4:
          map[x][y].image = water4;
          break;
        case 5:
          map[x][y].image = water5;
          break;
        case 6:
          map[x][y].image = water6;
          break;
        }
      map[x][y].ground = 0;
    }
  }

  return Weapons.Init();
}

void WorldSystem::StartMusic() {
  switch (miditoplay) {
    case 1:
      Sound.PlayMusic(attack1);
      break;
    case 2:
      Sound.PlayMusic(attack2);
      break;
    case 3:
      Sound.PlayMusic(combat1);
      break;
    case 4:
      Sound.PlayMusic(combat2);
      break;
    default:
      Sound.MusicStop();
      break;
  };
}

void WorldSystem::Update() {
  Player.SetView();
  Video.DrawMap(map);
  Parts.Update(); //Update the Particles Class
  Weapons.Update();
  Enemies.Update(Player.GetTrack());
  Player.Update();
}

// *** Editor Procs ***

int WorldSystem::Load(PACKFILE *mapdata) {
  char t;        //Check version
  pack_fread(&t, sizeof(char), mapdata);
  if (t != CurrFileVer)
    return 1;

  pack_fread(&miditoplay, sizeof(unsigned char), mapdata);
  Player.Load(mapdata);
  Enemies.Load(mapdata);
  unsigned char temp[mapx];
  int x, y;
  for (x = 0; x < mapx; x++) {
    pack_fread(temp, sizeof(unsigned char) * mapx, mapdata);
    for (y = 0; y < mapx; y++) {
      map[x][y].image = AtoD(temp[y]);
    }
  }

  return 0;
}

void WorldSystem::Save(PACKFILE *mapdata) {
  //Write Version
  pack_fwrite(&CurrFileVer, sizeof(char), mapdata);

  pack_fwrite(&miditoplay, sizeof(unsigned char), mapdata);
  Player.Save(mapdata);
  Enemies.Save(mapdata);
  unsigned char temp[mapx];
  int x, y;
  for (x = 0; x < mapx; x++) {
    for (y = 0; y < mapx; y++) {
      temp[y] = DtoA(map[x][y].image);
    }
    pack_fwrite(temp, sizeof(unsigned char) * mapx, mapdata);
  }
}

void WorldSystem::Draw(BITMAP *BUFFER, short int viewx, short int viewy) {
  int tempx, tempy;
  for (tempx = (viewx / 32); tempx <= ((viewx + mscrx - 1) / 32); tempx++) {
    for (tempy = (viewy / 32); tempy <= ((viewy + mscry - 1) / 32); tempy++) {
      blit((BITMAP *)data[map[tempx][tempy].image].dat, BUFFER, 0, 0, tempx * 32 - viewx, tempy * 32 - viewy, 32, 32);
    }
  }
  Enemies.Draw(BUFFER, viewx, viewy);
  Player.Draw(BUFFER, viewx, viewy);
}

void WorldSystem::AddTile(unsigned char x, unsigned char y, unsigned char image) {
  map[x][y].image = image;
}

unsigned char WorldSystem::GetTile(unsigned char x, unsigned char y) {
  return map[x][y].image;
}
